%% create data for the simulation experiments==============================
clear
clc
close all
%
m=3000; %samples
A=[unifrnd(-20,20,m,1) , unifrnd(-20,20,m,1) , unifrnd(-20,20,m,1)];
b=zeros(m,1); %preallocation
X=repmat([3 ; 12; 8],1,m/2); %true parameters
X=[X,repmat([6 ; 12; 8],1,m/2)]; %true parameters with step
for i=1:m
    b(i,1)=A(i,:)*X(:,i);
end
[~,n]=size(A); %n sum of inputs
[~,d]=size(b); %n sum of outputs
q=n+d;
p=4; %noise power in dBW
% setup with various noise
A1=A(:,1)+wgn(m,1,2*p); %add white Gaussian noise
A2=A(:,2)+wgn(m,1,p); %add white Gaussian noise
A3=A(:,3)+wgn(m,1,3*p); %add white Gaussian noise
A_err=(A-[A1 A2 A3]);
A=[A1, A2, A3]; %combine noisy and noise free inputs
%
b1=b+wgn(m,1,2.5*p); %add white Gaussian noise
b_err=(b-b1);
b=b1;
%==========================================================================

%% figure 2 RGTLS validation with batch TLS==================================
%
%batch TLS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
h = waitbar(0,'Please wait for batch TLS...');
tic
TLSdata=zeros(m,2*n); %preallocation
lambda=0.997; %batch TLS with forgetting factor
Wl=diag(lambda.^(m-(1:m))'); %exponential forgetting 
%
for i=1:m 
    if i <=q+1
        Aini = A(1:q+1,:);
        bini = b(1:q+1,:);
        [Xhat, covXhat, ~, ~, ~, ~, ~] = fcn_tls(Aini, bini);
    else
        [Xhat, covXhat, ~, ~, ~, ~, ~] = fcn_tls( Wl(m-i+1:m,m-i+1:m)*A(1:i,:) , Wl(m-i+1:m,m-i+1:m)*b(1:i,:) );
    end
TLSdata(i,:)=[Xhat' ,diag(covXhat)']; %save data
waitbar(i / m);
end
close(h)
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% RGTLS with W=I in the TLS sense%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
RGTLSdata=zeros(m,2*n); %preallocation
thresh=1e-10;
lambda=0.997; %forgetting factor
W=eye(q); %RGTLS in the TLS sense
for i=1:m
    if i <=q+1
        Aini = A(1:q+1,:);
        bini = b(1:q+1,:);
        [Xhat, covXhat, S, V, ~, ~, ~] = fcn_tls(Aini, bini);
        AtA = Aini'*Aini;
    else
    [Xhat, covXhat, S, V, AtA] = fcn_rgtls(...
    A(i,:),b(i,:), S, V, AtA, thresh, i, lambda, W);
    end
RGTLSdata(i,:)=[Xhat' ,diag(covXhat)']; %save data        
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% figure%%%%%%%%%%%
diff = TLSdata - RGTLSdata;
%
figure
subplot(2,1,1)
plot(1:m,diff(:,1),'k-',1:m,diff(:,2),'k--',1:m,diff(:,3),'k:');
ylim([-2.5e-3 1.5e-2]);
legend('Xhat(1)', 'Xhat(2)', 'Xhat(3)');
xlabel('Samples');
ylabel('Xhat_{TLS} - Xhat_{RGTLS}');
title('Fig.2(a)')
subplot(2,1,2)
plot(1:m,diff(:,4),'k-',1:m,diff(:,5),'k--',1:m,diff(:,6),'k:');
ylim([-2.5e-5 4e-4]);
legend('sigma^2 Xhat(1)', 'sigma^2 Xhat(2)', 'sigma^2 Xhat(3)');
xlabel('Samples');
ylabel('sigma^2_{TLS} - sigma^2_{RGTLS}');
title('Fig.2(b)')
%==========================================================================

%% figure 3 RGTLS and RLS==================================================
%
% RLS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
RLSdata=zeros(m,2*n); %preallocation
Xhat=zeros(n,1); %Initial Parameter RLS
covXhat=100*eye(n); %init RLS
lambda=0.997; %single forgetting factor
%
for i = 1:m
[Xhat,covXhat]=fcn_rls(A(i,:),b(i,:),Xhat, covXhat, lambda);
err_x=100*[(X(1,i)-Xhat(1))/X(1,i) , (X(2,i)-Xhat(2))/X(2,i) , (X(3,i)-Xhat(3))/X(3,i)]; %relative estimation error
RLSdata(i,:)=[err_x , diag(covXhat)']; %save data
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% RGTLS with known error covariance matrix W = cov([A_err,b_err])...
%best choice!!%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
RGTLSdata=zeros(m,2*n); %preallocation
thresh=1e-10;
lambda=0.997; %forgetting factor
W = cov([A_err,b_err]); %RGTLS with known error covariance matrix
for i=1:m
    if i <=q+1
        Aini = A(1:q+1,:);
        bini = b(1:q+1,:);
        [Xhat, covXhat, S, V, ~, ~, ~] = fcn_tls(Aini, bini);
        AtA = Aini'*Aini;
    else
    [Xhat, covXhat, S, V, AtA] = fcn_rgtls(...
    A(i,:),b(i,:), S, V, AtA, thresh, i, lambda, W);
    end
err_x=100*[(X(1,i)-Xhat(1))/X(1,i) , (X(2,i)-Xhat(2))/X(2,i) , (X(3,i)-Xhat(3))/X(3,i)]; %relative estimation error
RGTLSdata(i,:)=[err_x ,diag(covXhat)']; %save data        
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% figure%%%%%%%%%%%
%
figure
subplot(2,1,1)
plot(1:m,RGTLSdata(:,1),'k-',1:m,RGTLSdata(:,2),'k--',1:m,RGTLSdata(:,3),'k:');
ylim([-20 20]);
legend('Xhat_{RGTLS}(1)', 'Xhat_{RGTLS}(2)', 'Xhat_{RGTLS}(3)');
xlabel('Samples');
ylabel('rel.est.err. in %');
title('Fig.3(a)')
subplot(2,1,2)
plot(1:m,RLSdata(:,1),'k-',1:m,RLSdata(:,2),'k--',1:m,RLSdata(:,3),'k:');
ylim([-20 20]);
legend('Xhat_{RLS}(1)', 'Xhat_{RLS}(2)', 'Xhat_{RLS}(3)');
xlabel('Samples');
ylabel('rel.est.err. in %');
title('Fig.3(b)')
%==========================================================================

%% figure 4 RGTLS with RLS-like tuning and RLS=============================
%
% RLS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
RLSdata=zeros(m,2*n); %preallocation
Xhat=zeros(n,1); %Initial Parameter RLS
covXhat=100*eye(n); %init RLS
lambda=0.997; %single forgetting factor
%
for i = 1:m
[Xhat,covXhat]=fcn_rls(A(i,:),b(i,:),Xhat, covXhat, lambda);
err_x=100*[(X(1,i)-Xhat(1))/X(1,i) , (X(2,i)-Xhat(2))/X(2,i) , (X(3,i)-Xhat(3))/X(3,i)]; %relative estimation error
RLSdata(i,:)=[err_x , diag(covXhat)']; %save data
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% RGTLS with RLS-like error covariance matrix ...
% W = diag([1e-5 1e-5 1e-5 1e5])%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tic
RGTLSdata=zeros(m,2*n); %preallocation
thresh=1e-10;
lambda=0.997; %forgetting factor
W = diag([1e-5 1e-5 1e-5 1e5]); %RGTLS with RLS-like error covariance matrix
for i=1:m
    if i <=q+1
        Aini = A(1:q+1,:);
        bini = b(1:q+1,:);
        [Xhat, covXhat, S, V, ~, ~, ~] = fcn_tls(Aini, bini);
        AtA = Aini'*Aini;
    else
    [Xhat, covXhat, S, V, AtA] = fcn_rgtls(...
    A(i,:),b(i,:), S, V, AtA, thresh, i, lambda, W);
    end
err_x=100*[(X(1,i)-Xhat(1))/X(1,i) , (X(2,i)-Xhat(2))/X(2,i) , (X(3,i)-Xhat(3))/X(3,i)]; %relative estimation error
RGTLSdata(i,:)=[err_x ,diag(covXhat)']; %save data        
end
toc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% figure%%%%%%%%%%%
%
figure
subplot(2,1,1)
plot(1:m,RLSdata(:,1),'k-',1:m,RGTLSdata(:,1),'r-');
ylim([-20 20]);
legend('Xhat_{RLS}(1)', 'Xhat_{RGTLS}(1)');
xlabel('Samples');
ylabel('rel.est.err. in %');
title('Fig.4(a)')
subplot(2,1,2)
plot(1:m,RLSdata(:,2),'k--',1:m,RGTLSdata(:,2),'r--',...
    1:m,RLSdata(:,3),'k:',1:m,RGTLSdata(:,3),'r:');
ylim([-20 20]);
legend('Xhat_{RLS}(2)', 'Xhat_{RGTLS}(2)',...
    'Xhat_{RLS}(3)', 'Xhat_{RGTLS}(3)');
xlabel('Samples');
ylabel('rel.est.err. in %');
title('Fig.4(b)')
%==========================================================================