%==========================================================================
% check out the TLS solution
%
% Inputs:
% V-right orthogonal matrix size (n,n)
% A-system input size (m,n)
% B-system output size (m,d) with d:=1 for MISO systems
% Outputs:
% X-the system parameters size (n,d)
% Delta_A-korrections in A size (m,n)
% Delta_B-korrections in B size (m,d)
% misfit-scalar of the misfit

function [X, Delta_A, Delta_B, misfit] = fcn_TLSsolution (V, A, B)
%
[~, n] = size(A);
[~, d] = size(B);
q=n+d;
% proceed to checkout
V12 = V(1:n,q);
V22 = V(q,q);
X = -V12 / V22;
% Quelle: VanHuffel1989a
% "Algebraic connections between the least squares
% and total least squares problems" p.435
Delta_Z=[A,B]*[V12;V22]*[V12',V22']; %data corrections
            % Alternative:
            % Quelle: VanHuffel1991 p.35
            % bis auf 1e-14 gleich wie oben
            % Delta_Z=S(q,q)*U(:,q)*V(:,q)'; %Nachteil U Matrix wird bentigt
            % Quelle: Markovsky2007 p.2287            
Delta_A=Delta_Z(:,1:n); %extract data
Delta_B=Delta_Z(:,q); %extract data
misfit=norm([Delta_A,Delta_B],'fro'); %misfit
end