%==========================================================================
%
%#codegen
% Solves the linear equation AX=B using
% incremental truncated total least squares.
% Input:
% A - new system input data rows
% B - new system output data rows,
% S_old - old eigenvalues
% V_old - old right orthogonal matrix, 
% AtA_old - old input covariance matrix
% thresh - truncation threshold,
% m - current sample number
% lambda - forgetting factor size(1)
% W - error covariance Matrix size(n+d,n+d)
% special cases:
% when W=diagonal (the erros are uncorrelated but unequally sized)
% the solution reduces to weighted TLS, set W to diag(diag(W)) 
% when W=I=eye(n+d) (the erros are uncorrelated and equally sized)
% the solution reduces to the ordinary TLS estimate
%
% Output:
% X - the system parameters size(n,d),
% covX - parameter covariance matrix size(n,n)  not transformed!!,
% S_new - new eigenvalues  not transformed!!,
% V_new - new right orthogonal matrix size(n,n)  not transformed!!,
% AtA_new - new input covariance matrix size(n,n)  not transformed!!
% Delta_A - corrections size(m,n)
% Delta_B - correction size(m,d)

function [X, covX, S_new, V_new, AtA_new, Delta_A, Delta_B] = fcn_rgtls(...
    A, B, S_old, V_old, AtA_old, thresh, m, lambda, W)

[~, n] = size(A);
[~, d] = size(B);
q=n+d;

C=chol(W);
invC=C\eye(q); % is equal to inv(C)
C11=invC(1:n,1:n);
C12=invC(1:n,q);
C22=invC(q,q);

Z=[A,B] * invC; %transform data into space with equal noise variances
A=Z(:,1:n); %extract data
B=Z(:,q); %extract data

[X, covX, S_new, V_new, AtA_new, Delta_A, Delta_B] = fcn_rtls(...
    A, B, S_old, V_old, AtA_old, thresh, m, lambda);

X=(C11 * X -C12)/C22; %back transformation
%S_new=???  %back transformation
%V_new=???  %back transformation
%AtA_new=???  %back transformation
%covX???  %back transformation

Delta_Z=[Delta_A,Delta_B]*C; %back transformation
Delta_A=Delta_Z(:,1:n); %extract data
Delta_B=Delta_Z(:,q); %extract data
end
%==========================================================================