%==========================================================================
% Rekursive Least Squares Estimator for MISO Systems A*X=B
%
% Inputs:
% A_ini-initinal system input with at least q samples or empty []
% B_ini-initinal system output with at least q samples or empty []
% A-system input size (m,n)
% B-system output size (m,d) with d:=1 for single output
% X_old-old or initial parameter size (n,1)
% P_old-old or initial parameter covariance matrix size (n,n)
% lambda-forgetting factor
% switch size(lambda)
% case (1,1) standard single forgetting factor algorithm
% case (n,1) algorithm with multiple forgetting for each parameter
% m-current sample number
%
% Outputs:
% X_new-parameter update size (n,1)
% P_old-parameter covariance matrix update size (n,n)
%
% source RLS, standard single forgetting factor algorithm:
% "System identification : theory for the user" LJUNG1999
%
% source RLS, algorithm with multiple forgetting for each parameter:
% "Recursive least squares with forgetting for online...
...estimation of vehicle mass and road grade" Vahidi2005


function [X_new, P_new] =  fcn_rls(A, B, X_old, P_old, lambda)
%
[~, n] = size(A);
[i , ~] = size(lambda);
%
%==================from here RLS======================================
if i==n; % RLS algorithm with multiple forgetting for each parameter        
    L_new =((P_old * A') ./ lambda) / ... 
    (1 + sum( (P_old * A'.^2) ./ lambda) ); % Berechnen von Korrekturvektor L
    X_new = (X_old + L_new * (B - A * X_old)); % Parameterschtzwerte X
    % aktualisieren der P-Matrix
    P_new = diag(diag( (eye(n) - L_new * A) * P_old) ./ lambda);        
elseif i==1; % RLS algorithm with standard single forgetting factor        
    L_new= (P_old * A') / (lambda + A * P_old * A'); % Berechnen von L 
    X_new = X_old + L_new * (B - A * X_old); % Parameterschtzwerte X
    % aktualisieren der P-Matrix   
    P_new = (eye(n) - L_new * A) * P_old / lambda;    
else
    error('wrong size of lambda')      
end

end %of main function======================================================



