%==========================================================================
% incremental SVD update

% Input:
% A - new system input data size(1,n)
% B - new system output data size(1,d)
% S_old - old eigenvalues size(q,q)
% V_old - old right orthogonal matrix size(q,q)
% thresh - truncation threshold
% lambda - forgetting factor size(1) for single forgetting or size(q,1) for
% multiple forgetting
%
% Output:
% S_new - new eigenvalues  size(q,q)
% V_new - new right orthogonal matrix  size(q,q)


% source: GU 1993 "A stable and fast algorithm for updating the singular value decomposition"

function [S_new, V_new] = fcn_rsvd(A, B, S_old, V_old, thresh, lambda)
%

[~, n] = size(A);
[~, d] = size(B);
q=n+d;

%from here incremental TLS
z=[A,B]';
a=V_old'*z;
M=[ (diag(lambda)*S_old);...
a'];
[~, K]=qr(z-V_old*a);
v=sqrt(det(K'*K));
    if v<thresh %take eps 1e-16 as minimum
    [~, N, Q]=svd(M);
    S_new=N(1:q,1:q); %truncation
    else
    [~, S_new, Q]=svd(M,'econ'); 
    end                                           
V_new=(V_old*Q); %update V 
%==========================================================================