%==========================================================================
% Solves the linear equation AX=B using
% incremental truncated total least squares.
% Input:
% A - new system input data rows
% B - new system output data rows,
% S_old - old eigenvalues
% V-old right orthogonal matrix
% AtA_old - old input covariance matrix
% thresh - truncation threshold,
% m - current sample number
% lambda - forgetting factor size(1) for single forgetting or size(q,1) for
% multiple forgetting
% Output:
% X - the system parameters
% covX - parameter covariance matrix size(n,n)
% S_new - new eigenvalues
% V_new - new right orthogonal matrix
% AtA_new - new input covariance matrix
% Delta_A-korrections in A size (m,n)
% Delta_B-korrections in B size (m,d)

function [X, covX, S_new, V_new, AtA_new, Delta_A, Delta_B] = fcn_rtls(...
    A, B, S_old, V_old, AtA_old, thresh, m, lambda)
%
[~, n] = size(A);
[~, d] = size(B);
q=n+d;
  
[S_new, V_new] = fcn_rsvd(A, B, S_old, V_old, thresh, lambda); %recursive SVD      
[X, Delta_A, Delta_B, ~] = fcn_TLSsolution (V_new, A, B); %get the TLS solution    
%Parameter Kovarianz aus [VanHuffel1991 S.242] iterativ 
AtA_new = ((AtA_old * lambda^2) + (A'*A)); %update AtA
s_quad=S_new(q,q)^2/m;
covX = (1+ norm(X)^2)*s_quad * ...
    ((AtA_new - m*s_quad*eye(n)) \ eye(n)); %parameter covariance   

end
%==========================================================================

