%==========================================================================
% Solves the linear equation AX=B using total least-squares for MISO
% systems
% Inputs:
% A-system input size (m,n)
% B-system output size (m,d) with d:=1 for MISO systems
% Outputs:
% X-the system parameters size (n,d)
% covX-parameter variance size (n,n)
% S-matrix of eigen values size (n,n)
% V-right orthogonal matrix size (n,n)
% Delta_A-korrections in A size (m,n)
% Delta_B-korrections in B size (m,d)
% misfit-scalar of the misfit
% U*S*V is the svd of [A, B] where U is not assigned

function [X, covX, S, V, Delta_A, Delta_B, misfit] = fcn_tls(A,B)
%
[m, n] = size(A);
[~, d] = size(B);
q=n+d;
% augmented matrix
Z = [A, B];
[~, S, V]=svd( Z );
[X, Delta_A, Delta_B, misfit] = fcn_TLSsolution (V, A, B);       
%Parameter Kovarianz aus [VanHuffel1991 S.242] 
AtA=A'*A;
s_quad=S(q,q)^2/m;
covX=(1+ norm(X)^2)*s_quad *...
    ((AtA - m*s_quad*eye(n)) \ eye(n)); %parameter covariance
end
%==========================================================================

